@@ -1,11 +1,10 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2022, by Samuel Williams.
+# Copyright, 2022-2025, by Samuel Williams.
 # Copyright, 2023, by Genki Takiuchi.
 
-require 'async/io/buffer'
-require 'protocol/http/body/stream'
+require "protocol/http/body/stream"
 
 module Protocol
 	module Rack
@@ -31,8 +30,6 @@ module Protocol
 			
 			include Protocol::HTTP::Body::Stream::Reader
 			
-			alias gets read_partial
-			
 			# Enumerate chunks of the request body.
 			# @yields {|chunk| ...}
 			# 	@parameter chunk [String]
@@ -67,8 +64,10 @@ module Protocol
 				if @body and @body.respond_to?(:rewind)
 					# If the body is not rewindable, this will fail.
 					@body.rewind
+					
 					@buffer = nil
 					@finished = false
+					@closed = false
 					
 					return true
 				end
@@ -90,7 +89,25 @@ module Protocol
 			
 			def read_next
 				if @body
-					@body.read
+					# User's may forget to call #close...
+					if chunk = @body.read
+						# If the user reads exactly the content length, we close the stream automatically:
+						# https://github.com/socketry/async-http/issues/183
+						if @body.empty?
+							@body.close
+							@closed = true
+						end
+						
+						return chunk
+					else
+						unless @closed
+							# So if we are at the end of the stream, we close it automatically:
+							@body.close
+							@closed = true
+						end
+						
+						return nil
+					end
 				elsif @closed
 					raise IOError, "Stream is not readable, input has been closed!"
 				end
